package java.awt.image;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.Graphics2D;
import sun.awt.image.ImagingLib;
import java.awt.RenderingHints;

public class ConvolveOp implements BufferedImageOp, RasterOp
{
    Kernel kernel;
    int edgeHint;
    RenderingHints hints;
    public static final int EDGE_ZERO_FILL = 0;
    public static final int EDGE_NO_OP = 1;
    
    public ConvolveOp(Kernel kernel, int edgeHint, RenderingHints hints) {
        this.kernel = kernel;
        this.edgeHint = edgeHint;
        this.hints = hints;
    }
    
    public ConvolveOp(Kernel kernel) {
        this.kernel = kernel;
        this.edgeHint = 0;
    }
    
    public int getEdgeCondition() {
        return this.edgeHint;
    }
    
    public final Kernel getKernel() {
        return (Kernel)this.kernel.clone();
    }
    
    @Override
    public final BufferedImage filter(BufferedImage convertToIntDiscrete, BufferedImage dst) {
        if (convertToIntDiscrete == null) {
            throw new NullPointerException("src image is null");
        }
        if (convertToIntDiscrete == dst) {
            throw new IllegalArgumentException("src image cannot be the same as the dst image");
        }
        boolean n = false;
        ColorModel colorModel = convertToIntDiscrete.getColorModel();
        if ((colorModel instanceof IndexColorModel)) {
            convertToIntDiscrete = ((IndexColorModel)colorModel).convertToIntDiscrete(convertToIntDiscrete.getRaster(), false);
            colorModel = convertToIntDiscrete.getColorModel();
        }
        if (dst == null) {
            dst = this.createCompatibleDestImage(convertToIntDiscrete, null);
        }
        else {
            ColorModel colorModel2 = dst.getColorModel();
            if (colorModel.getColorSpace().getType() != colorModel2.getColorSpace().getType()) {
                n = true;
                dst = this.createCompatibleDestImage(convertToIntDiscrete, null);
                dst.getColorModel();
            }
            else if ((colorModel2 instanceof IndexColorModel)) {
                dst = this.createCompatibleDestImage(convertToIntDiscrete, null);
                dst.getColorModel();
            }
        }
        if (ImagingLib.filter(this, convertToIntDiscrete, dst) == null) {
            throw new ImagingOpException("Unable to convolve src image");
        }
        if (!(n == 0)) {
            new ColorConvertOp(this.hints).filter(dst, dst);
        }
        else if (dst != dst) {
            Graphics2D graphics = dst.createGraphics();
            try {
                graphics.drawImage(dst, 0, 0, null);
            }
            finally {
                graphics.dispose();
            }
        }
        return dst;
    }
    
    @Override
    public final WritableRaster filter(Raster src, WritableRaster compatibleDestRaster) {
        if (compatibleDestRaster == null) {
            compatibleDestRaster = this.createCompatibleDestRaster(src);
        }
        else {
            if (src == compatibleDestRaster) {
                throw new IllegalArgumentException("src image cannot be the same as the dst image");
            }
            if (src.getNumBands() != compatibleDestRaster.getNumBands()) {
                throw new ImagingOpException("Different number of bands in src  and dst Rasters");
            }
        }
        if (ImagingLib.filter(this, src, compatibleDestRaster) == null) {
            throw new ImagingOpException("Unable to convolve src image");
        }
        return compatibleDestRaster;
    }
    
    @Override
    public BufferedImage createCompatibleDestImage(BufferedImage bufferedImage, ColorModel colorModel) {
        int width = bufferedImage.getWidth();
        int height = bufferedImage.getHeight();
        WritableRaster writableRaster = null;
        if (colorModel == null) {
            colorModel = bufferedImage.getColorModel();
            if ((colorModel instanceof IndexColorModel)) {
                colorModel = ColorModel.getRGBdefault();
            }
            else {
                writableRaster = bufferedImage.getData().createCompatibleWritableRaster(width, height);
            }
        }
        if (writableRaster == null) {
            writableRaster = colorModel.createCompatibleWritableRaster(width, height);
        }
        return new BufferedImage(colorModel, writableRaster, colorModel.isAlphaPremultiplied(), null);
    }
    
    @Override
    public WritableRaster createCompatibleDestRaster(Raster raster) {
        return raster.createCompatibleWritableRaster();
    }
    
    @Override
    public final Rectangle2D getBounds2D(BufferedImage bufferedImage) {
        return this.getBounds2D(bufferedImage.getRaster());
    }
    
    @Override
    public final Rectangle2D getBounds2D(Raster raster) {
        return raster.getBounds();
    }
    
    @Override
    public final Point2D getPoint2D(Point2D point2D, Point2D point2D2) {
        if (point2D2 == null) {
            point2D2 = new Point2D.Float();
        }
        point2D2.setLocation(point2D.getX(), point2D.getY());
        return point2D2;
    }
    
    @Override
    public final RenderingHints getRenderingHints() {
        return this.hints;
    }
}
